@@ -4,6 +4,7 @@ import (
 	goctx "context"
 	"fmt"
 	"os"
+	"reflect"
 	"regexp"
 	"testing"
 	"time"
@@ -65,47 +66,43 @@ func TestLocalStorageOperator(t *testing.T) {
 	}
 
 	selectedNode := selectNode(t, f.KubeClient)
+
+	originalNodeTaints := selectedNode.Spec.Taints
+	selectedNode.Spec.Taints = []v1.Taint{
+		{
+			Key:    "localstorage",
+			Value:  "testvalue",
+			Effect: "NoSchedule",
+		},
+	}
+	updatedNode, err := waitForNodeTaintUpdate(t, f.KubeClient, selectedNode, retryInterval, timeout)
+	if err != nil {
+		t.Fatalf("error tainting node : %v", err)
+	}
+	selectedNode = updatedNode
+
+	defer func() {
+		selectedNode.Spec.Taints = originalNodeTaints
+		selectedNode, err = waitForNodeTaintUpdate(t, f.KubeClient, selectedNode, retryInterval, timeout)
+		if err != nil {
+			t.Fatalf("error restoring original taints on node: %v", err)
+		}
+	}()
+
 	selectedDisk, _ := selectDisk(f.KubeClient, selectedNode)
 	waitForPVCreation := true
 	if selectedDisk == "" {
 		waitForPVCreation = false
 		selectedDisk = "/dev/foobar"
 	}
-	localVolume := &localv1.LocalVolume{
-		TypeMeta: metav1.TypeMeta{
-			Kind:       "LocalVolume",
-			APIVersion: localv1.SchemeGroupVersion.String(),
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      "test-local-disk",
-			Namespace: namespace,
-		},
-		Spec: localv1.LocalVolumeSpec{
-			NodeSelector: &v1.NodeSelector{
-				NodeSelectorTerms: []v1.NodeSelectorTerm{
-					{
-						MatchFields: []v1.NodeSelectorRequirement{
-							{Key: "metadata.name", Operator: v1.NodeSelectorOpIn, Values: []string{selectedNode.Name}},
-						},
-					},
-				},
-			},
-			StorageClassDevices: []localv1.StorageClassDevice{
-				{
-					StorageClassName: "test-local-sc",
-					DevicePaths:      []string{selectedDisk},
-				},
-			},
-		},
-	}
 
-	err = f.Client.Create(goctx.TODO(), localVolume, nil)
+	localVolume := getFakeLocalVolume(selectedNode, selectedDisk, namespace)
+
+	err = f.Client.Create(goctx.TODO(), localVolume, &framework.CleanupOptions{TestContext: ctx, Timeout: cleanupTimeout, RetryInterval: cleanupRetryInterval})
 	if err != nil {
 		t.Fatalf("error creating localvolume cr : %v", err)
 	}
 
-	defer deleteLocalVolume(localVolume, f.Client)
-
 	provisionerDSName := localVolume.Name + "-local-provisioner"
 	diskMakerDSName := localVolume.Name + "-local-diskmaker"
 
@@ -124,6 +121,16 @@ func TestLocalStorageOperator(t *testing.T) {
 		t.Fatalf("error verifying localvolume cr: %v", err)
 	}
 
+	err = verifyDaemonSetTolerations(f.KubeClient, provisionerDSName, namespace, localVolume.Spec.Tolerations)
+	if err != nil {
+		t.Fatalf("error verifying provisioner tolerations match localvolume: %v", err)
+	}
+
+	err = verifyDaemonSetTolerations(f.KubeClient, diskMakerDSName, namespace, localVolume.Spec.Tolerations)
+	if err != nil {
+		t.Fatalf("error verifying diskmaker tolerations match localvolume: %v", err)
+	}
+
 	err = checkLocalVolumeStatus(localVolume)
 	if err != nil {
 		t.Fatalf("error checking localvolume condition: %v", err)
@@ -139,15 +146,6 @@ func TestLocalStorageOperator(t *testing.T) {
 			t.Errorf("error deleting created PV: %v", err)
 		}
 	}
-	err = deleteLocalVolume(localVolume, f.Client)
-	if err != nil {
-		t.Fatalf("error deleting localvolume: %v", err)
-	}
-
-	err = verifyStorageClassDeletion(localVolume.Spec.StorageClassDevices[0].StorageClassName, f.KubeClient)
-	if err != nil {
-		t.Fatalf("error verifying storageClass cleanup: %v", err)
-	}
 }
 
 func verifyLocalVolume(lv *localv1.LocalVolume, client framework.FrameworkClient) error {
@@ -169,43 +167,26 @@ func verifyLocalVolume(lv *localv1.LocalVolume, client framework.FrameworkClient
 	return waitErr
 }
 
-func deleteLocalVolume(lv *localv1.LocalVolume, client framework.FrameworkClient) error {
-	err := client.Delete(goctx.TODO(), lv)
-	if err != nil {
-		if apierrors.IsNotFound(err) {
-			return nil
+func verifyDaemonSetTolerations(kubeclient kubernetes.Interface, daemonSetName, namespace string, tolerations []v1.Toleration) error {
+	dsTolerations := []v1.Toleration{}
+	err := wait.Poll(retryInterval, timeout, func() (done bool, err error) {
+		daemonset, err := kubeclient.AppsV1().DaemonSets(namespace).Get(daemonSetName, metav1.GetOptions{})
+		if err != nil {
+			if apierrors.IsNotFound(err) {
+				return false, nil
+			}
+			return false, err
 		}
+		dsTolerations = daemonset.Spec.Template.Spec.Tolerations
+		return true, err
+	})
+	if err != nil {
 		return err
 	}
-	waitErr := wait.PollImmediate(retryInterval, timeout, func() (bool, error) {
-		objectKey := dynclient.ObjectKey{
-			Namespace: lv.Namespace,
-			Name:      lv.Name,
-		}
-		err := client.Get(goctx.TODO(), objectKey, lv)
-		if err != nil {
-			if apierrors.IsNotFound(err) {
-				return true, nil
-			}
-			return false, err
-		}
-		return false, nil
-	})
-	return waitErr
-}
-
-func verifyStorageClassDeletion(scName string, kubeclient kubernetes.Interface) error {
-	waitError := wait.Poll(retryInterval, timeout, func() (done bool, err error) {
-		_, err = kubeclient.StorageV1().StorageClasses().Get(scName, metav1.GetOptions{})
-		if err != nil {
-			if apierrors.IsNotFound(err) {
-				return true, nil
-			}
-			return false, err
-		}
-		return false, nil
-	})
-	return waitError
+	if !reflect.DeepEqual(dsTolerations, tolerations) {
+		return fmt.Errorf("toleration mismatch between daemonset and localvolume: %v, %v", dsTolerations, tolerations)
+	}
+	return nil
 }
 
 func checkLocalVolumeStatus(lv *localv1.LocalVolume) error {
@@ -372,3 +353,63 @@ func waitForDaemonSet(t *testing.T, kubeclient kubernetes.Interface, namespace,
 	t.Logf("Daemonset available (%d/%d)\n", nodeCount, nodeCount)
 	return nil
 }
+
+func waitForNodeTaintUpdate(t *testing.T, kubeclient kubernetes.Interface, node v1.Node, retryInterval, timeout time.Duration) (v1.Node, error) {
+	var err error
+	var newNode *v1.Node
+	name := node.Name
+	err = wait.Poll(retryInterval, timeout, func() (done bool, err error) {
+		newNode, err = kubeclient.Core().Nodes().Get(name, metav1.GetOptions{})
+		newNode.Spec.Taints = node.Spec.Taints
+		newNode, err = kubeclient.Core().Nodes().Update(newNode)
+		if err != nil {
+			t.Logf("Failed to update node %v successfully : %v", name, err)
+			return false, nil
+		}
+		return true, nil
+	})
+	if err != nil {
+		return *newNode, err
+	}
+	t.Logf("Node %v updated successfully\n", name)
+	return *newNode, nil
+}
+
+func getFakeLocalVolume(selectedNode v1.Node, selectedDisk, namespace string) *localv1.LocalVolume {
+	localVolume := &localv1.LocalVolume{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "LocalVolume",
+			APIVersion: localv1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "test-local-disk",
+			Namespace: namespace,
+		},
+		Spec: localv1.LocalVolumeSpec{
+			NodeSelector: &v1.NodeSelector{
+				NodeSelectorTerms: []v1.NodeSelectorTerm{
+					{
+						MatchFields: []v1.NodeSelectorRequirement{
+							{Key: "metadata.name", Operator: v1.NodeSelectorOpIn, Values: []string{selectedNode.Name}},
+						},
+					},
+				},
+			},
+			Tolerations: []v1.Toleration{
+				{
+					Key:      "localstorage",
+					Value:    "testvalue",
+					Operator: "Equal",
+				},
+			},
+			StorageClassDevices: []localv1.StorageClassDevice{
+				{
+					StorageClassName: "test-local-sc",
+					DevicePaths:      []string{selectedDisk},
+				},
+			},
+		},
+	}
+
+	return localVolume
+}
